package com.danitheskunk.skunkgame;

import com.danitheskunk.skunkworks.BaseGame;
import com.danitheskunk.skunkworks.Vec2i;
import com.danitheskunk.skunkworks.WindowStretchMode;
import com.danitheskunk.skunkworks.gfx.Color;
import com.danitheskunk.skunkworks.gfx.IRenderContext;
import com.danitheskunk.skunkworks.gfx.vt.Terminal;

import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.List;

public class SkunkGame extends BaseGame {
	private Terminal term;
	private double timeSinceLastUpdate;
	private double timePerUpdate;
	public static Dictionary<String, Double> res;
	private List<Building> buildings;
	public static Vec2i mousePos;
	private List<Button> actionButtons;

	public SkunkGame() {
		super(new Vec2i(1280, 700), "SkunkGame v0.0.0");
		window.setStretchMode(WindowStretchMode.ASPECT);
		var font = window.loadFontTileset("fonts\\ega-8x14.png");
		term = new Terminal(new Vec2i(160, 50), font);
		res = new Hashtable<>();
		buildings = new ArrayList<>();
		timePerUpdate = 1.0;
		actionButtons = new ArrayList<>();

		res.put("wood", 0.0);

		addBuilding(
			"den",
			"Den",
			new CostElement[]{new CostElement("wood", 10)}
		);

		var woodButton = new Button(new Vec2i(31, 0), "Gather Wood");
		woodButton.onClick(() -> addRes("wood", 1.0));
		actionButtons.add(woodButton);


	}

	private void drawScreen() {
		term.clear(Color.BLACK);
		drawResources();
		term.drawVerticalSingleLine(new Vec2i(30, 0), term.getSize().getY());

		var buttons = new ArrayList<Button>();
		buttons.addAll(actionButtons);
		for(var building : buildings) {
			if(building.isButtonVisible()) {
				buttons.add(building.getButton());
			}
		}
		int i = 0;
		for(var button : buttons) {
			button.setPos(Vec2i.add(new Vec2i(31, 0),
				Vec2i.mul(new Vec2i(0, 3), i)
			));
			button.render(term);
			++i;
		}
	}

	private void drawResources() {
		term.drawString(new Vec2i(0, 0), "Wood:");
		term.drawString(new Vec2i(18, 0),
			String.format("%d", (int) getRes("wood"))
		);
	}

	public static double getRes(String name) {
		return res.get(name);
	}

	public static void setRes(String name, double count) {
		res.put(name, count);
	}

	public static void addRes(String name, double count) {
		setRes(name, getRes(name) + count);
	}

	private void addBuilding(
		String name, String displayName, CostElement[] cost
	) {
		buildings.add(new Building(name, displayName, new Cost(cost)));
	}

	private Building getBuilding(String name) {
		//todo: optimise
		for(var building : buildings) {
			if(building.getName().equals(name)) {
				return building;
			}
		}
		return null;
	}


	private void tick() {

	}

	@Override
	protected void render(IRenderContext rc) {
		drawScreen();
		rc.drawTerminal(term);
	}

	@Override
	protected void update(double delta) {
		timeSinceLastUpdate -= delta;
		if(timeSinceLastUpdate <= 0.0) {
			tick();
			timeSinceLastUpdate += timePerUpdate;
		}
		mousePos = Vec2i.div(window.getMousePos(), term.getFullCharSize());
		if(window.isMouseClicked(0)) {
			for(var button : actionButtons) {
				button.processClick(mousePos);
			}
			for(var building : buildings) {
				if(building.isButtonVisible()) {
					building.getButton().processClick(mousePos);
				}
			}
		}
	}

	public static void main(String[] args) {
		System.out.println("Hello world <3");
		new SkunkGame().run();
	}
}
